﻿// Wintab Gamepad
// OKOROM SOFT
// ver 1.2.0.0

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using vJoyInterfaceWrap;
using System.Runtime.InteropServices;
using System.Windows.Interop;
using System.Windows.Forms;
using System.IO;

namespace WintabGamepad
{
    /// <summary>
    /// MainWindow.xaml の相互作用ロジック
    /// </summary>
    public partial class MainWindow : Window
    {
        // iniファイル用設定
        public static string iniFileName = AppDomain.CurrentDomain.BaseDirectory + "WintabGamepad.ini";
        [DllImport("kernel32.dll")]
        public static extern uint GetPrivateProfileString(string lpAppName, string lpKeyName, string lpDefault, StringBuilder lpReturnedString, uint nSize, string lpFileName);

        // フォーカスを取らないウィンドウの設定
        [DllImport("user32.dll")]
        private static extern IntPtr SetWindowLong(IntPtr hWnd, int nIndex, int dwNewLong);
        [DllImport("user32.dll")]
        private static extern int GetWindowLong(IntPtr hWnd, int nIndex);

        private const int GWL_EXSTYLE = -20;
        private const int WS_EX_NOACTIVATE = 0x08000000;

        protected override void OnSourceInitialized(EventArgs e)
        {
            base.OnSourceInitialized(e);
            var helper = new WindowInteropHelper(this);
            SetWindowLong(helper.Handle, GWL_EXSTYLE, GetWindowLong(helper.Handle, GWL_EXSTYLE) | WS_EX_NOACTIVATE);
        }

        // vJoy変数定義
        public vJoy joystick;
        public vJoy.JoystickState iReport;
        public uint id;

        // ボタン色用変数定義
        public SolidColorBrush brush_nonact_stroke;     // 非アクティブ時 枠線
        public SolidColorBrush brush_nonact_fill;       // 非アクティブ時 塗りつぶし
        public SolidColorBrush brush_act_stroke;        // アクティブ時 枠線
        public SolidColorBrush brush_act_fill;          // アクティブ時 塗りつぶし

        // メインウィンドウのタッチ&マウス操作スルー用変数定義
        public bool through_enable;   // スルー設定
        public SolidColorBrush brush_through_enable;    // スルー設定時背景色
        public SolidColorBrush brush_through_disable;   // 非スルー設定時背景色

        // パッド用変数の定義
        private double analog_l_radius;
        private byte analog_l_sensitivity;
        private double analog_r_radius;
        private byte analog_r_sensitivity;
        private double dpad_radius;
        private double dpad_distance;
        private double dpad_angle;

        // 表示・非表示トグル用変数の定義
        private bool show_gamepad;

        // NotificationIcon定義
        private System.Windows.Forms.NotifyIcon notif;

        // コンストラクタ
        public MainWindow()
        {
            // デフォルト動作
            InitializeComponent();

            // iniファイル存在チェック
            if(!File.Exists(iniFileName))
            {
                OpenFileDialog ofd = new OpenFileDialog();
                ofd.Filter = "iniファイル|*.ini";
                ofd.FileName = "";
                ofd.Multiselect = false;
                ofd.CheckFileExists = true;
                ofd.DefaultExt = "*.ini";
                ofd.Title = "iniファイルの選択";
                ofd.InitialDirectory = AppDomain.CurrentDomain.BaseDirectory;
                if (ofd.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                {
                    iniFileName = ofd.FileName;
                }
                else
                {
                    System.Windows.MessageBox.Show("iniファイルが選択されませんでした");
                    this.Close();
                    return;
                }
            }
            
            // ボタンの設定
            if(!configure_buttons())
            {
                System.Windows.MessageBox.Show("iniファイルの内容が正しくありません");
                this.Close();
                return;
            }
            
            // vJoy変数初期化
            StringBuilder sb = new StringBuilder(1024);
            GetPrivateProfileString("vJoy_Config", "TargetDevice", "1", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
            id = (uint)int.Parse(sb.ToString());
            joystick = new vJoy();
            iReport = new vJoy.JoystickState();

            // 有効なvJoyドライバの有無を確認
            if (!joystick.vJoyEnabled())
            {
                System.Windows.MessageBox.Show("vJoyドライバが無効になっています");
                this.Close();
                return;
            }

            // vJoyデバイスの占有を取得
            joystick.AcquireVJD(id);
            VjdStat status = joystick.GetVJDStatus(id);
            if (status != VjdStat.VJD_STAT_OWN)
            {
                System.Windows.MessageBox.Show("vJoyデバイスは他のプログラムによって使用されています");
                this.Close();
                return;
            }

            // ボタン状態のリセット
            // Rzは0x4000(中間値)にセット
            joystick.ResetVJD(id);
            joystick.SetAxis(0x4000, id, HID_USAGES.HID_USAGE_RZ);

            // 表示トグル設定
            show_gamepad = true;

            // NotificationIconの設定
            notif = new NotifyIcon();
            notif.Text = "WintabGamepad";
            notif.Icon = Properties.Resources.Icon1;
            notif.Visible = true;
            notif.Click += new EventHandler(notif_Click);
        }

        // デストラクタ
        ~MainWindow()
        {
            // デバイス占有を破棄
            if (joystick != null)
            {
                if (joystick.GetVJDStatus(id) == VjdStat.VJD_STAT_OWN)
                {
                    joystick.RelinquishVJD(id);
                }
            }
        }

        // ボタンのコンフィグ
        private bool configure_buttons()
        {
            // iniファイル読み込み
            byte red, green, blue, thickness;
            float opacity;
            double analog_l_X, analog_l_Y
                , analog_r_X, analog_r_Y
                , btn1_radius, btn1_X, btn1_Y
                , btn2_radius, btn2_X, btn2_Y
                , btn3_radius, btn3_X, btn3_Y
                , btn4_radius, btn4_X, btn4_Y
                , btn5_radius, btn5_X, btn5_Y
                , btn6_radius, btn6_X, btn6_Y
                , btn7_radius, btn7_X, btn7_Y
                , btn8_radius, btn8_X, btn8_Y
                , btn9_radius, btn9_X, btn9_Y
                , btn10_radius, btn10_X, btn10_Y
                , btn11_radius, btn11_X, btn11_Y
                , btn12_radius, btn12_X, btn12_Y
                , dpad_X, dpad_Y;
            bool analog_l_enable, analog_r_enable
                , btn1_enable, btn2_enable, btn3_enable
                , btn4_enable, btn5_enable, btn6_enable
                , btn7_enable, btn8_enable, btn9_enable
                , btn10_enable, btn11_enable, btn12_enable
                , dpad_enable, dpad_cont_enable;
            StringBuilder sb = new StringBuilder(1024);

            try
            {
                // Main_Config
                GetPrivateProfileString("Main_Config", "Enable_Through", "true", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                through_enable = bool.Parse(sb.ToString());
                brush_through_enable = new SolidColorBrush(Color.FromArgb(0, 255, 255, 255));
                brush_through_disable = new SolidColorBrush(Color.FromArgb(1, 255, 255, 255));

                // Button_Color_Config
                GetPrivateProfileString("Button_Color_Config", "Stroke_Thickness", "2", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                thickness = (byte)int.Parse(sb.ToString());

                GetPrivateProfileString("Button_Color_Config", "NonAct_Stroke_R", "0", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                red = (byte)int.Parse(sb.ToString());
                GetPrivateProfileString("Button_Color_Config", "NonAct_Stroke_G", "0", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                green = (byte)int.Parse(sb.ToString());
                GetPrivateProfileString("Button_Color_Config", "NonAct_Stroke_B", "0", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                blue = (byte)int.Parse(sb.ToString());
                brush_nonact_stroke = new SolidColorBrush(Color.FromRgb(red, green, blue));
                GetPrivateProfileString("Button_Color_Config", "NonAct_Stroke_Opacity", "0.8", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                opacity = float.Parse(sb.ToString());
                opacity = Math.Min(Math.Max(float.Parse(sb.ToString()), 0), 1);
                brush_nonact_stroke.Opacity = opacity;

                GetPrivateProfileString("Button_Color_Config", "NonAct_Fill_R", "0", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                red = (byte)int.Parse(sb.ToString());
                GetPrivateProfileString("Button_Color_Config", "NonAct_Fill_G", "0", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                green = (byte)int.Parse(sb.ToString());
                GetPrivateProfileString("Button_Color_Config", "NonAct_Fill_B", "0", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                blue = (byte)int.Parse(sb.ToString());
                brush_nonact_fill = new SolidColorBrush(Color.FromRgb(red, green, blue));
                GetPrivateProfileString("Button_Color_Config", "NonAct_Fill_Opacity", "0.2", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                opacity = Math.Min(Math.Max(float.Parse(sb.ToString()), 0), 1);
                brush_nonact_fill.Opacity = opacity;

                GetPrivateProfileString("Button_Color_Config", "Act_Stroke_R", "0", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                red = (byte)int.Parse(sb.ToString());
                GetPrivateProfileString("Button_Color_Config", "Act_Stroke_G", "0", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                green = (byte)int.Parse(sb.ToString());
                GetPrivateProfileString("Button_Color_Config", "Act_Stroke_B", "0", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                blue = (byte)int.Parse(sb.ToString());
                brush_act_stroke = new SolidColorBrush(Color.FromRgb(red, green, blue));
                GetPrivateProfileString("Button_Color_Config", "Act_Stroke_Opacity", "0.8", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                opacity = Math.Min(Math.Max(float.Parse(sb.ToString()), 0), 1);
                brush_act_stroke.Opacity = opacity;

                GetPrivateProfileString("Button_Color_Config", "Act_Fill_R", "0", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                red = (byte)int.Parse(sb.ToString());
                GetPrivateProfileString("Button_Color_Config", "Act_Fill_G", "0", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                green = (byte)int.Parse(sb.ToString());
                GetPrivateProfileString("Button_Color_Config", "Act_Fill_B", "0", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                blue = (byte)int.Parse(sb.ToString());
                brush_act_fill = new SolidColorBrush(Color.FromRgb(red, green, blue));
                GetPrivateProfileString("Button_Color_Config", "Act_Fill_Opacity", "0.2", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                opacity = Math.Min(Math.Max(float.Parse(sb.ToString()), 0), 1);
                brush_act_fill.Opacity = opacity;

                // Analog_L_Config
                GetPrivateProfileString("Analog_L_Config", "Analog_L_Enable", "false", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                analog_l_enable = bool.Parse(sb.ToString());
                GetPrivateProfileString("Analog_L_Config", "Analog_L_Radius", "100", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                analog_l_radius = double.Parse(sb.ToString());
                GetPrivateProfileString("Analog_L_Config", "Analog_L_X", "110", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                analog_l_X = double.Parse(sb.ToString());
                GetPrivateProfileString("Analog_L_Config", "Analog_L_Y", "500", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                analog_l_Y = double.Parse(sb.ToString());
                GetPrivateProfileString("Analog_L_Config", "Analog_L_Sensitivity", "80", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                analog_l_sensitivity = (byte)Math.Max(Math.Min(int.Parse(sb.ToString()), 100), 0);

                // Analog_R_Config
                GetPrivateProfileString("Analog_R_Config", "Analog_R_Enable", "false", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                analog_r_enable = bool.Parse(sb.ToString());
                GetPrivateProfileString("Analog_R_Config", "Analog_R_Radius", "100", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                analog_r_radius = double.Parse(sb.ToString());
                GetPrivateProfileString("Analog_R_Config", "Analog_R_X", "1170", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                analog_r_X = double.Parse(sb.ToString());
                GetPrivateProfileString("Analog_R_Config", "Analog_R_Y", "500", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                analog_r_Y = double.Parse(sb.ToString());
                GetPrivateProfileString("Analog_R_Config", "Analog_R_Sensitivity", "80", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                analog_r_sensitivity = (byte)Math.Max(Math.Min(int.Parse(sb.ToString()), 100), 0);

                // btn1_Config
                GetPrivateProfileString("Button1_Config", "Button1_Enable", "false", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                btn1_enable = bool.Parse(sb.ToString());
                GetPrivateProfileString("Button1_Config", "Button1_Radius", "40", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                btn1_radius = double.Parse(sb.ToString());
                GetPrivateProfileString("Button1_Config", "Button1_X", "1120", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                btn1_X = double.Parse(sb.ToString());
                GetPrivateProfileString("Button1_Config", "Button1_Y", "330", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                btn1_Y = double.Parse(sb.ToString());

                // btn2_Config
                GetPrivateProfileString("Button2_Config", "Button2_Enable", "false", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                btn2_enable = bool.Parse(sb.ToString());
                GetPrivateProfileString("Button2_Config", "Button2_Radius", "40", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                btn2_radius = double.Parse(sb.ToString());
                GetPrivateProfileString("Button2_Config", "Button2_X", "1220", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                btn2_X = double.Parse(sb.ToString());
                GetPrivateProfileString("Button2_Config", "Button2_Y", "330", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                btn2_Y = double.Parse(sb.ToString());

                // btn3_Config
                GetPrivateProfileString("Button3_Config", "Button3_Enable", "false", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                btn3_enable = bool.Parse(sb.ToString());
                GetPrivateProfileString("Button3_Config", "Button3_Radius", "40", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                btn3_radius = double.Parse(sb.ToString());
                GetPrivateProfileString("Button3_Config", "Button3_X", "1120", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                btn3_X = double.Parse(sb.ToString());
                GetPrivateProfileString("Button3_Config", "Button3_Y", "240", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                btn3_Y = double.Parse(sb.ToString());

                // btn4_Config
                GetPrivateProfileString("Button4_Config", "Button4_Enable", "false", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                btn4_enable = bool.Parse(sb.ToString());
                GetPrivateProfileString("Button4_Config", "Button4_Radius", "40", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                btn4_radius = double.Parse(sb.ToString());
                GetPrivateProfileString("Button4_Config", "Button4_X", "1220", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                btn4_X = double.Parse(sb.ToString());
                GetPrivateProfileString("Button4_Config", "Button4_Y", "240", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                btn4_Y = double.Parse(sb.ToString());

                // btn5_Config
                GetPrivateProfileString("Button5_Config", "Button5_Enable", "false", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                btn5_enable = bool.Parse(sb.ToString());
                GetPrivateProfileString("Button5_Config", "Button5_Radius", "40", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                btn5_radius = double.Parse(sb.ToString());
                GetPrivateProfileString("Button5_Config", "Button5_X", "60", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                btn5_X = double.Parse(sb.ToString());
                GetPrivateProfileString("Button5_Config", "Button5_Y", "130", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                btn5_Y = double.Parse(sb.ToString());

                // btn6_Config
                GetPrivateProfileString("Button6_Config", "Button6_Enable", "false", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                btn6_enable = bool.Parse(sb.ToString());
                GetPrivateProfileString("Button6_Config", "Button6_Radius", "40", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                btn6_radius = double.Parse(sb.ToString());
                GetPrivateProfileString("Button6_Config", "Button6_X", "160", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                btn6_X = double.Parse(sb.ToString());
                GetPrivateProfileString("Button6_Config", "Button6_Y", "130", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                btn6_Y = double.Parse(sb.ToString());

                // btn7_Config
                GetPrivateProfileString("Button7_Config", "Button7_Enable", "false", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                btn7_enable = bool.Parse(sb.ToString());
                GetPrivateProfileString("Button7_Config", "Button7_Radius", "40", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                btn7_radius = double.Parse(sb.ToString());
                GetPrivateProfileString("Button7_Config", "Button7_X", "1120", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                btn7_X = double.Parse(sb.ToString());
                GetPrivateProfileString("Button7_Config", "Button7_Y", "130", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                btn7_Y = double.Parse(sb.ToString());

                // btn8_Config
                GetPrivateProfileString("Button8_Config", "Button8_Enable", "false", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                btn8_enable = bool.Parse(sb.ToString());
                GetPrivateProfileString("Button8_Config", "Button8_Radius", "40", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                btn8_radius = double.Parse(sb.ToString());
                GetPrivateProfileString("Button8_Config", "Button8_X", "1220", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                btn8_X = double.Parse(sb.ToString());
                GetPrivateProfileString("Button8_Config", "Button8_Y", "130", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                btn8_Y = double.Parse(sb.ToString());

                // btn9_Config
                GetPrivateProfileString("Button9_Config", "Button9_Enable", "false", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                btn9_enable = bool.Parse(sb.ToString());
                GetPrivateProfileString("Button9_Config", "Button9_Radius", "40", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                btn9_radius = double.Parse(sb.ToString());
                GetPrivateProfileString("Button9_Config", "Button9_X", "60", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                btn9_X = double.Parse(sb.ToString());
                GetPrivateProfileString("Button9_Config", "Button9_Y", "670", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                btn9_Y = double.Parse(sb.ToString());

                // btn10_Config
                GetPrivateProfileString("Button10_Config", "Button10_Enable", "false", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                btn10_enable = bool.Parse(sb.ToString());
                GetPrivateProfileString("Button10_Config", "Button10_Radius", "40", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                btn10_radius = double.Parse(sb.ToString());
                GetPrivateProfileString("Button10_Config", "Button10_X", "160", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                btn10_X = double.Parse(sb.ToString());
                GetPrivateProfileString("Button10_Config", "Button10_Y", "670", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                btn10_Y = double.Parse(sb.ToString());

                // btn11_Config
                GetPrivateProfileString("Button11_Config", "Button11_Enable", "false", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                btn11_enable = bool.Parse(sb.ToString());
                GetPrivateProfileString("Button11_Config", "Button11_Radius", "40", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                btn11_radius = double.Parse(sb.ToString());
                GetPrivateProfileString("Button11_Config", "Button11_X", "1120", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                btn11_X = double.Parse(sb.ToString());
                GetPrivateProfileString("Button11_Config", "Button11_Y", "670", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                btn11_Y = double.Parse(sb.ToString());

                // btn12_Config
                GetPrivateProfileString("Button12_Config", "Button12_Enable", "false", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                btn12_enable = bool.Parse(sb.ToString());
                GetPrivateProfileString("Button12_Config", "Button12_Radius", "40", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                btn12_radius = double.Parse(sb.ToString());
                GetPrivateProfileString("Button12_Config", "Button12_X", "1220", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                btn12_X = double.Parse(sb.ToString());
                GetPrivateProfileString("Button12_Config", "Button12_Y", "670", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                btn12_Y = double.Parse(sb.ToString());

                // dpad_Config
                GetPrivateProfileString("Dpad_Config", "Dpad_Enable", "false", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                dpad_enable = bool.Parse(sb.ToString());
                GetPrivateProfileString("Dpad_Config", "Dpad_Continuous_Enable", "false", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                dpad_cont_enable = bool.Parse(sb.ToString());
                GetPrivateProfileString("Dpad_Config", "Dpad_Radius", "100", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                dpad_radius = double.Parse(sb.ToString());
                GetPrivateProfileString("Dpad_Config", "Dpad_X", "110", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                dpad_X = double.Parse(sb.ToString());
                GetPrivateProfileString("Dpad_Config", "Dpad_Y", "290", sb, (uint)Convert.ToInt32(sb.Capacity), iniFileName);
                dpad_Y = double.Parse(sb.ToString());
            }
            catch
            {
                return false;
            }

            // 操作スルー設定
            if(through_enable)
            {
                main.Background = brush_through_enable;
            }
            else
            {
                main.Background = brush_through_disable;
            }

            // ボタン設定
            analog_l.Stroke = brush_nonact_stroke;
            analog_l.Fill = brush_nonact_fill;
            analog_l.StrokeThickness = thickness;
            analog_l.SetValue(Canvas.LeftProperty, analog_l_X - analog_l_radius);
            analog_l.SetValue(Canvas.TopProperty, analog_l_Y - analog_l_radius);
            analog_l.Width = analog_l_radius * 2;
            analog_l.Height = analog_l_radius * 2;
            if (!analog_l_enable)
            {
                analog_l.Visibility = System.Windows.Visibility.Collapsed;
            }

            analog_r.Stroke = brush_nonact_stroke;
            analog_r.Fill = brush_nonact_fill;
            analog_r.StrokeThickness = thickness;
            analog_r.SetValue(Canvas.LeftProperty, analog_r_X - analog_r_radius);
            analog_r.SetValue(Canvas.TopProperty, analog_r_Y - analog_r_radius);
            analog_r.Width = analog_r_radius * 2;
            analog_r.Height = analog_r_radius * 2;
            if (!analog_r_enable)
            {
                analog_r.Visibility = System.Windows.Visibility.Collapsed;
            }
            
            btn1.Stroke = brush_nonact_stroke;
            btn1.Fill = brush_nonact_fill;
            btn1.StrokeThickness = thickness;
            btn1.SetValue(Canvas.LeftProperty, btn1_X - btn1_radius);
            btn1.SetValue(Canvas.TopProperty, btn1_Y - btn1_radius);
            btn1.Width = btn1_radius * 2;
            btn1.Height = btn1_radius * 2;
            if (!btn1_enable)
            {
                btn1.Visibility = System.Windows.Visibility.Collapsed;
            }
            
            btn2.Stroke = brush_nonact_stroke;
            btn2.Fill = brush_nonact_fill;
            btn2.StrokeThickness = thickness;
            btn2.SetValue(Canvas.LeftProperty, btn2_X - btn2_radius);
            btn2.SetValue(Canvas.TopProperty, btn2_Y - btn2_radius);
            btn2.Width = btn2_radius * 2;
            btn2.Height = btn2_radius * 2;
            if (!btn2_enable)
            {
                btn2.Visibility = System.Windows.Visibility.Collapsed;
            }
            
            btn3.Stroke = brush_nonact_stroke;
            btn3.Fill = brush_nonact_fill;
            btn3.StrokeThickness = thickness;
            btn3.SetValue(Canvas.LeftProperty, btn3_X - btn3_radius);
            btn3.SetValue(Canvas.TopProperty, btn3_Y - btn3_radius);
            btn3.Width = btn3_radius * 2;
            btn3.Height = btn3_radius * 2;
            if (!btn3_enable)
            {
                btn3.Visibility = System.Windows.Visibility.Collapsed;
            }
            
            btn4.Stroke = brush_nonact_stroke;
            btn4.Fill = brush_nonact_fill;
            btn4.StrokeThickness = thickness;
            btn4.SetValue(Canvas.LeftProperty, btn4_X - btn4_radius);
            btn4.SetValue(Canvas.TopProperty, btn4_Y - btn4_radius);
            btn4.Width = btn4_radius * 2;
            btn4.Height = btn4_radius * 2;
            if (!btn4_enable)
            {
                btn4.Visibility = System.Windows.Visibility.Collapsed;
            }
            
            btn5.Stroke = brush_nonact_stroke;
            btn5.Fill = brush_nonact_fill;
            btn5.StrokeThickness = thickness;
            btn5.SetValue(Canvas.LeftProperty, btn5_X - btn5_radius);
            btn5.SetValue(Canvas.TopProperty, btn5_Y - btn5_radius);
            btn5.Width = btn5_radius * 2;
            btn5.Height = btn5_radius * 2;
            if (!btn5_enable)
            {
                btn5.Visibility = System.Windows.Visibility.Collapsed;
            }
            
            btn6.Stroke = brush_nonact_stroke;
            btn6.Fill = brush_nonact_fill;
            btn6.StrokeThickness = thickness;
            btn6.SetValue(Canvas.LeftProperty, btn6_X - btn6_radius);
            btn6.SetValue(Canvas.TopProperty, btn6_Y - btn6_radius);
            btn6.Width = btn6_radius * 2;
            btn6.Height = btn6_radius * 2;
            if (!btn6_enable)
            {
                btn6.Visibility = System.Windows.Visibility.Collapsed;
            }
            
            btn7.Stroke = brush_nonact_stroke;
            btn7.Fill = brush_nonact_fill;
            btn7.StrokeThickness = thickness;
            btn7.SetValue(Canvas.LeftProperty, btn7_X - btn7_radius);
            btn7.SetValue(Canvas.TopProperty, btn7_Y - btn7_radius);
            btn7.Width = btn7_radius * 2;
            btn7.Height = btn7_radius * 2;
            if (!btn7_enable)
            {
                btn7.Visibility = System.Windows.Visibility.Collapsed;
            }

            btn8.Stroke = brush_nonact_stroke;
            btn8.Fill = brush_nonact_fill;
            btn8.StrokeThickness = thickness;
            btn8.SetValue(Canvas.LeftProperty, btn8_X - btn8_radius);
            btn8.SetValue(Canvas.TopProperty, btn8_Y - btn8_radius);
            btn8.Width = btn8_radius * 2;
            btn8.Height = btn8_radius * 2;
            if (!btn8_enable)
            {
                btn8.Visibility = System.Windows.Visibility.Collapsed;
            }

            btn9.Stroke = brush_nonact_stroke;
            btn9.Fill = brush_nonact_fill;
            btn9.StrokeThickness = thickness;
            btn9.SetValue(Canvas.LeftProperty, btn9_X - btn9_radius);
            btn9.SetValue(Canvas.TopProperty, btn9_Y - btn9_radius);
            btn9.Width = btn9_radius * 2;
            btn9.Height = btn9_radius * 2;
            if (!btn9_enable)
            {
                btn9.Visibility = System.Windows.Visibility.Collapsed;
            }

            btn10.Stroke = brush_nonact_stroke;
            btn10.Fill = brush_nonact_fill;
            btn10.StrokeThickness = thickness;
            btn10.SetValue(Canvas.LeftProperty, btn10_X - btn10_radius);
            btn10.SetValue(Canvas.TopProperty, btn10_Y - btn10_radius);
            btn10.Width = btn10_radius * 2;
            btn10.Height = btn10_radius * 2;
            if (!btn10_enable)
            {
                btn10.Visibility = System.Windows.Visibility.Collapsed;
            }
            
            btn11.Stroke = brush_nonact_stroke;
            btn11.Fill = brush_nonact_fill;
            btn11.StrokeThickness = thickness;
            btn11.SetValue(Canvas.LeftProperty, btn11_X - btn11_radius);
            btn11.SetValue(Canvas.TopProperty, btn11_Y - btn11_radius);
            btn11.Width = btn11_radius * 2;
            btn11.Height = btn11_radius * 2;
            if (!btn11_enable)
            {
                btn11.Visibility = System.Windows.Visibility.Collapsed;
            }

            btn12.Stroke = brush_nonact_stroke;
            btn12.Fill = brush_nonact_fill;
            btn12.StrokeThickness = thickness;
            btn12.SetValue(Canvas.LeftProperty, btn12_X - btn12_radius);
            btn12.SetValue(Canvas.TopProperty, btn12_Y - btn12_radius);
            btn12.Width = btn12_radius * 2;
            btn12.Height = btn12_radius * 2;
            if (!btn12_enable)
            {
                btn12.Visibility = System.Windows.Visibility.Collapsed;
            }

            dpad_ccl.Stroke = brush_nonact_stroke;
            dpad_ccl.StrokeThickness = thickness;
            dpad_ccl.SetValue(Canvas.LeftProperty, dpad_X - dpad_radius * 0.3);
            dpad_ccl.SetValue(Canvas.TopProperty, dpad_Y - dpad_radius * 0.3);
            dpad_ccl.Width = dpad_radius * 0.6;
            dpad_ccl.Height = dpad_radius * 0.6;

            dpad_au.Stroke = brush_nonact_stroke;
            dpad_al.Stroke = brush_nonact_stroke;
            dpad_ad.Stroke = brush_nonact_stroke;
            dpad_ar.Stroke = brush_nonact_stroke;
            dpad_au.StrokeThickness = thickness;
            dpad_al.StrokeThickness = thickness;
            dpad_ad.StrokeThickness = thickness;
            dpad_ar.StrokeThickness = thickness;
            dpad_au.SetValue(Canvas.LeftProperty, dpad_X);
            dpad_au.SetValue(Canvas.TopProperty, dpad_Y - dpad_radius * 0.8);
            dpad_al.SetValue(Canvas.LeftProperty, dpad_X - dpad_radius * 0.8);
            dpad_al.SetValue(Canvas.TopProperty, dpad_Y);
            dpad_ad.SetValue(Canvas.LeftProperty, dpad_X);
            dpad_ad.SetValue(Canvas.TopProperty, dpad_Y + dpad_radius * 0.8);
            dpad_ar.SetValue(Canvas.LeftProperty, dpad_X + dpad_radius * 0.8);
            dpad_ar.SetValue(Canvas.TopProperty, dpad_Y);
            dpad_au.Points = PointCollection.Parse("0,0 " + dpad_radius * -0.15 + "," + dpad_radius * 0.2 + " " + dpad_radius * 0.15 + "," + dpad_radius * 0.2);
            dpad_al.Points = PointCollection.Parse("0,0 " + dpad_radius * 0.2 + "," + dpad_radius * -0.15 + " " + dpad_radius * 0.2 + "," + dpad_radius * 0.15);
            dpad_ad.Points = PointCollection.Parse("0,0 " + dpad_radius * -0.15 + "," + dpad_radius * -0.2 + " " + dpad_radius * 0.15 + "," + dpad_radius * -0.2);
            dpad_ar.Points = PointCollection.Parse("0,0 " + dpad_radius * -0.2 + "," + dpad_radius * -0.15 + " " + dpad_radius * -0.2 + "," + dpad_radius * 0.15);

            dpad.Stroke = brush_nonact_stroke;
            dpad.Fill = brush_nonact_fill;
            dpad.StrokeThickness = thickness;
            dpad.SetValue(Canvas.LeftProperty, dpad_X - dpad_radius);
            dpad.SetValue(Canvas.TopProperty, dpad_Y - dpad_radius);
            dpad.Width = dpad_radius * 2;
            dpad.Height = dpad_radius * 2;
            if (!dpad_enable || !dpad_cont_enable)
            {
                dpad.Visibility = System.Windows.Visibility.Collapsed;
                dpad_ccl.Visibility = System.Windows.Visibility.Collapsed;
                dpad_au.Visibility = System.Windows.Visibility.Collapsed;
                dpad_al.Visibility = System.Windows.Visibility.Collapsed;
                dpad_ad.Visibility = System.Windows.Visibility.Collapsed;
                dpad_ar.Visibility = System.Windows.Visibility.Collapsed;
            }

            double dpad_rad_xy = Math.Sqrt(2) * (dpad_radius - 10) / 2;
            
            dpad_u.Stroke = brush_nonact_stroke;
            dpad_u.Fill = brush_nonact_fill;
            dpad_u.StrokeThickness = thickness;
            dpad_u.SetValue(Canvas.LeftProperty, dpad_X);
            dpad_u.SetValue(Canvas.TopProperty, dpad_Y);
            dpad_u.Data = Geometry.Parse("M 0,-10 L " + (dpad_rad_xy * -1).ToString() + "," + (dpad_rad_xy * -1 - 10).ToString() + " S 0," + (dpad_radius * -1.2 - 10).ToString() + " " + (dpad_rad_xy).ToString() + "," + (dpad_rad_xy * -1 - 10).ToString() + " L 0,-10");
            if (!dpad_enable || dpad_cont_enable)
            {
                dpad_u.Visibility = System.Windows.Visibility.Collapsed;
            }
            
            dpad_l.Stroke = brush_nonact_stroke;
            dpad_l.Fill = brush_nonact_fill;
            dpad_l.StrokeThickness = thickness;
            dpad_l.SetValue(Canvas.LeftProperty, dpad_X);
            dpad_l.SetValue(Canvas.TopProperty, dpad_Y);
            dpad_l.Data = Geometry.Parse("M -10,0 L " + (dpad_rad_xy * -1 - 10).ToString() + "," + (dpad_rad_xy * -1).ToString() + " S " + (dpad_radius * -1.2 - 10).ToString() + ",0 " + (dpad_rad_xy * -1 - 10).ToString() + "," + (dpad_rad_xy).ToString() + " L -10,0");
            if (!dpad_enable || dpad_cont_enable)
            {
                dpad_l.Visibility = System.Windows.Visibility.Collapsed;
            }

            dpad_d.Stroke = brush_nonact_stroke;
            dpad_d.Fill = brush_nonact_fill;
            dpad_d.StrokeThickness = thickness;
            dpad_d.SetValue(Canvas.LeftProperty, dpad_X);
            dpad_d.SetValue(Canvas.TopProperty, dpad_Y);
            dpad_d.Data = Geometry.Parse("M 0,10 L " + (dpad_rad_xy * -1).ToString() + "," + (dpad_rad_xy + 10).ToString() + " S 0," + (dpad_radius * 1.2 + 10).ToString() + " " + (dpad_rad_xy).ToString() + "," + (dpad_rad_xy + 10).ToString() + " L 0,10");
            if (!dpad_enable || dpad_cont_enable)
            {
                dpad_d.Visibility = System.Windows.Visibility.Collapsed;
            }

            dpad_r.Stroke = brush_nonact_stroke;
            dpad_r.Fill = brush_nonact_fill;
            dpad_r.StrokeThickness = thickness;
            dpad_r.SetValue(Canvas.LeftProperty, dpad_X);
            dpad_r.SetValue(Canvas.TopProperty, dpad_Y);
            dpad_r.Data = Geometry.Parse("M 10,0 L " + (dpad_rad_xy + 10).ToString() + "," + (dpad_rad_xy * -1).ToString() + " S " + (dpad_radius * 1.2 + 10).ToString() + ",0 " + (dpad_rad_xy + 10).ToString() + "," + (dpad_rad_xy).ToString() + " L 10,0");
            if (!dpad_enable || dpad_cont_enable)
            {
                dpad_r.Visibility = System.Windows.Visibility.Collapsed;
            }
                        
            btn_menu.Stroke = brush_nonact_stroke;
            btn_menu.Fill = brush_nonact_fill;

            return true;
        }
        
        private void notif_Click(object sender, EventArgs e)
        {
            MessageBoxResult result = System.Windows.MessageBox.Show("WintabGamepadを終了します", "WintabGamepad", MessageBoxButton.OKCancel);
            if(result == MessageBoxResult.OK)
            {
                notif.Dispose();
                this.Close();
            }
        }

        // 2点間の距離を求める
        public double get_distance(double x1, double y1, double x2, double y2)
        {
            return Math.Sqrt(Math.Pow((x2 - x1), 2) + Math.Pow((y2 - y1), 2));
        }

        // 2点を結ぶ直線とx=x1とが交わる角度を求める(符号は処理で必要な用に調整)
        public double get_angle(double x1, double y1, double x2, double y2)
        {
            return Math.Atan2(x1 - x2, y2 - y1) / Math.PI * 180 + 180;
        }

        private void analog_r_TouchMove(object sender, TouchEventArgs e)
        {
            joystick.SetAxis((int)Math.Max(Math.Min(((e.GetTouchPoint(analog_r).Position.X - analog_r_radius) * (int)(analog_r_sensitivity * 500 / analog_r_radius) + 0x4000), 0x8000), 0), id, HID_USAGES.HID_USAGE_Z);
            joystick.SetAxis((int)Math.Max(Math.Min(((e.GetTouchPoint(analog_r).Position.Y - analog_r_radius) * (int)(analog_r_sensitivity * 500 / analog_r_radius) + 0x4000), 0x8000), 0), id, HID_USAGES.HID_USAGE_RZ);
        }

        private void analog_r_TouchUp(object sender, TouchEventArgs e)
        {
            analog_r.ReleaseTouchCapture(e.TouchDevice);
            analog_r.Stroke = brush_nonact_stroke;
            analog_r.Fill = brush_nonact_fill;
            joystick.SetAxis(0x4000, id, HID_USAGES.HID_USAGE_Z);
            joystick.SetAxis(0x4000, id, HID_USAGES.HID_USAGE_RZ);
        }

        private void analog_r_TouchDown(object sender, TouchEventArgs e)
        {
            analog_r.CaptureTouch(e.TouchDevice);
            analog_r.Stroke = brush_act_stroke;
            analog_r.Fill = brush_act_fill;
        }

        private void analog_l_TouchMove(object sender, TouchEventArgs e)
        {
            joystick.SetAxis((int)Math.Max(Math.Min(((e.GetTouchPoint(analog_l).Position.X - analog_l_radius) * (int)(analog_l_sensitivity * 500 / analog_l_radius) + 0x4000), 0x8000), 0), id, HID_USAGES.HID_USAGE_X);
            joystick.SetAxis((int)Math.Max(Math.Min(((e.GetTouchPoint(analog_l).Position.Y - analog_l_radius) * (int)(analog_l_sensitivity * 500 / analog_l_radius) + 0x4000), 0x8000), 0), id, HID_USAGES.HID_USAGE_Y);
        }

        private void analog_l_TouchUp(object sender, TouchEventArgs e)
        {
            analog_l.ReleaseTouchCapture(e.TouchDevice);
            analog_l.Stroke = brush_nonact_stroke;
            analog_l.Fill = brush_nonact_fill;
            joystick.SetAxis(0x4000, id, HID_USAGES.HID_USAGE_X);
            joystick.SetAxis(0x4000, id, HID_USAGES.HID_USAGE_Y);
        }

        private void analog_l_TouchDown(object sender, TouchEventArgs e)
        {
            analog_l.CaptureTouch(e.TouchDevice);
            analog_l.Stroke = brush_act_stroke;
            analog_l.Fill = brush_act_fill;
        }

        private void dpad_TouchDown(object sender, TouchEventArgs e)
        {
            dpad.CaptureTouch(e.TouchDevice);
            dpad.Stroke = brush_act_stroke;
            dpad.Fill = brush_act_fill;
            dpad_ccl.Stroke = brush_act_stroke;
            dpad_au.Stroke = brush_act_stroke;
            dpad_al.Stroke = brush_act_stroke;
            dpad_ad.Stroke = brush_act_stroke;
            dpad_ar.Stroke = brush_act_stroke;
        }

        private void dpad_TouchUp(object sender, TouchEventArgs e)
        {
            dpad.ReleaseTouchCapture(e.TouchDevice);
            dpad.Stroke = brush_nonact_stroke;
            dpad.Fill = brush_nonact_fill;
            dpad_ccl.Stroke = brush_nonact_stroke;
            dpad_au.Stroke = brush_nonact_stroke;
            dpad_al.Stroke = brush_nonact_stroke;
            dpad_ad.Stroke = brush_nonact_stroke;
            dpad_ar.Stroke = brush_nonact_stroke;
            dpad_au.Opacity = 1;
            dpad_al.Opacity = 1;
            dpad_ad.Opacity = 1;
            dpad_ar.Opacity = 1;
            joystick.SetContPov(-1, id, 1);
        }

        private void dpad_TouchMove(object sender, TouchEventArgs e)
        {
            dpad_distance = get_distance(dpad_radius, dpad_radius, e.GetTouchPoint(dpad).Position.X, e.GetTouchPoint(dpad).Position.Y);
            dpad_angle = get_angle(dpad_radius, dpad_radius, e.GetTouchPoint(dpad).Position.X, e.GetTouchPoint(dpad).Position.Y);
            if(dpad_distance < dpad_radius * 0.3) // 中心から30%以内の場合はニュートラル
            {
                joystick.SetContPov(-1, id, 1);
                dpad_au.Opacity = 0;
                dpad_al.Opacity = 0;
                dpad_ad.Opacity = 0;
                dpad_ar.Opacity = 0;
            }
            else
            {
                // 角度を8方向に固定
                if (dpad_angle <= 22.5 || dpad_angle > 337.5)   // 上
                {
                    joystick.SetContPov(0, id, 1);
                    dpad_au.Opacity = 1;
                    dpad_al.Opacity = 0.2;
                    dpad_ad.Opacity = 0.2;
                    dpad_ar.Opacity = 0.2;
                }
                else if (22.5 < dpad_angle && dpad_angle <= 67.5)    // 右上
                {
                    joystick.SetContPov(4500, id, 1);
                    dpad_au.Opacity = 1;
                    dpad_al.Opacity = 0.2;
                    dpad_ad.Opacity = 0.2;
                    dpad_ar.Opacity = 1;
                }
                else if (67.5 < dpad_angle && dpad_angle <= 112.5)   // 右
                {
                    joystick.SetContPov(9000, id, 1);
                    dpad_au.Opacity = 0.2;
                    dpad_al.Opacity = 0.2;
                    dpad_ad.Opacity = 0.2;
                    dpad_ar.Opacity = 1;
                }
                else if (112.5 < dpad_angle && dpad_angle <= 157.5) // 右下
                {
                    joystick.SetContPov(13500, id, 1);
                    dpad_au.Opacity = 0.2;
                    dpad_al.Opacity = 0.2;
                    dpad_ad.Opacity = 1;
                    dpad_ar.Opacity = 1;
                }
                else if (157.5 < dpad_angle && dpad_angle <= 202.5) // 下
                {
                    joystick.SetContPov(18000, id, 1);
                    dpad_au.Opacity = 0.2;
                    dpad_al.Opacity = 0.2;
                    dpad_ad.Opacity = 1;
                    dpad_ar.Opacity = 0.2;
                }
                else if (202.5 < dpad_angle && dpad_angle <= 247.5) // 左下
                {
                    joystick.SetContPov(22500, id, 1);
                    dpad_au.Opacity = 0.2;
                    dpad_al.Opacity = 1;
                    dpad_ad.Opacity = 1;
                    dpad_ar.Opacity = 0.2;
                }
                else if (247.5 < dpad_angle && dpad_angle <= 292.5) // 左
                {
                    joystick.SetContPov(27000, id, 1);
                    dpad_au.Opacity = 0.2;
                    dpad_al.Opacity = 1;
                    dpad_ad.Opacity = 0.2;
                    dpad_ar.Opacity = 0.2;
                }
                else if (292.5 < dpad_angle && dpad_angle <= 337.5) // 左上
                {
                    joystick.SetContPov(31500, id, 1);
                    dpad_au.Opacity = 1;
                    dpad_al.Opacity = 1;
                    dpad_ad.Opacity = 0.2;
                    dpad_ar.Opacity = 0.2;
                }
            }
        }

        private void btn1_TouchEnter(object sender, TouchEventArgs e)
        {
            btn1.CaptureTouch(e.TouchDevice);
            btn1.Stroke = brush_act_stroke;
            btn1.Fill = brush_act_fill;
            joystick.SetBtn(true, id, 1);
        }

        private void btn1_TouchLeave(object sender, TouchEventArgs e)
        {
            btn1.ReleaseTouchCapture(e.TouchDevice);
            btn1.Stroke = brush_nonact_stroke;
            btn1.Fill = brush_nonact_fill;
            joystick.SetBtn(false, id, 1);
        }

        private void btn2_TouchEnter(object sender, TouchEventArgs e)
        {
            btn2.CaptureTouch(e.TouchDevice);
            btn2.Stroke = brush_act_stroke;
            btn2.Fill = brush_act_fill;
            joystick.SetBtn(true, id, 2);
        }

        private void btn2_TouchLeave(object sender, TouchEventArgs e)
        {
            btn2.ReleaseTouchCapture(e.TouchDevice);
            btn2.Stroke = brush_nonact_stroke;
            btn2.Fill = brush_nonact_fill;
            joystick.SetBtn(false, id, 2);
        }

        private void btn3_TouchEnter(object sender, TouchEventArgs e)
        {
            btn3.CaptureTouch(e.TouchDevice);
            btn3.Stroke = brush_act_stroke;
            btn3.Fill = brush_act_fill;
            joystick.SetBtn(true, id, 3);
        }

        private void btn3_TouchLeave(object sender, TouchEventArgs e)
        {
            btn3.ReleaseTouchCapture(e.TouchDevice);
            btn3.Stroke = brush_nonact_stroke;
            btn3.Fill = brush_nonact_fill;
            joystick.SetBtn(false, id, 3);
        }

        private void btn4_TouchEnter(object sender, TouchEventArgs e)
        {
            btn4.CaptureTouch(e.TouchDevice);
            btn4.Stroke = brush_act_stroke;
            btn4.Fill = brush_act_fill;
            joystick.SetBtn(true, id, 4);
        }

        private void btn4_TouchLeave(object sender, TouchEventArgs e)
        {
            btn4.ReleaseTouchCapture(e.TouchDevice);
            btn4.Stroke = brush_nonact_stroke;
            btn4.Fill = brush_nonact_fill;
            joystick.SetBtn(false, id, 4);
        }

        private void btn5_TouchEnter(object sender, TouchEventArgs e)
        {
            btn5.CaptureTouch(e.TouchDevice);
            btn5.Stroke = brush_act_stroke;
            btn5.Fill = brush_act_fill;
            joystick.SetBtn(true, id, 5);
        }

        private void btn5_TouchLeave(object sender, TouchEventArgs e)
        {
            btn5.ReleaseTouchCapture(e.TouchDevice);
            btn5.Stroke = brush_nonact_stroke;
            btn5.Fill = brush_nonact_fill;
            joystick.SetBtn(false, id, 5);
        }

        private void btn6_TouchEnter(object sender, TouchEventArgs e)
        {
            btn6.CaptureTouch(e.TouchDevice);
            btn6.Stroke = brush_act_stroke;
            btn6.Fill = brush_act_fill;
            joystick.SetBtn(true, id, 6);
        }

        private void btn6_TouchLeave(object sender, TouchEventArgs e)
        {
            btn6.ReleaseTouchCapture(e.TouchDevice);
            btn6.Stroke = brush_nonact_stroke;
            btn6.Fill = brush_nonact_fill;
            joystick.SetBtn(false, id, 6);
        }

        private void btn7_TouchEnter(object sender, TouchEventArgs e)
        {
            btn7.CaptureTouch(e.TouchDevice);
            btn7.Stroke = brush_act_stroke;
            btn7.Fill = brush_act_fill;
            joystick.SetBtn(true, id, 7);
        }

        private void btn7_TouchLeave(object sender, TouchEventArgs e)
        {
            btn7.ReleaseTouchCapture(e.TouchDevice);
            btn7.Stroke = brush_nonact_stroke;
            btn7.Fill = brush_nonact_fill;
            joystick.SetBtn(false, id, 7);
        }

        private void btn8_TouchEnter(object sender, TouchEventArgs e)
        {
            btn8.CaptureTouch(e.TouchDevice);
            btn8.Stroke = brush_act_stroke;
            btn8.Fill = brush_act_fill;
            joystick.SetBtn(true, id, 8);
        }

        private void btn8_TouchLeave(object sender, TouchEventArgs e)
        {
            btn8.ReleaseTouchCapture(e.TouchDevice);
            btn8.Stroke = brush_nonact_stroke;
            btn8.Fill = brush_nonact_fill;
            joystick.SetBtn(false, id, 8);
        }

        private void btn9_TouchEnter(object sender, TouchEventArgs e)
        {
            btn9.CaptureTouch(e.TouchDevice);
            btn9.Stroke = brush_act_stroke;
            btn9.Fill = brush_act_fill;
            joystick.SetBtn(true, id, 9);
        }

        private void btn9_TouchLeave(object sender, TouchEventArgs e)
        {
            btn9.ReleaseTouchCapture(e.TouchDevice);
            btn9.Stroke = brush_nonact_stroke;
            btn9.Fill = brush_nonact_fill;
            joystick.SetBtn(false, id, 9);
        }

        private void btn10_TouchEnter(object sender, TouchEventArgs e)
        {
            btn10.CaptureTouch(e.TouchDevice);
            btn10.Stroke = brush_act_stroke;
            btn10.Fill = brush_act_fill;
            joystick.SetBtn(true, id, 10);
        }

        private void btn10_TouchLeave(object sender, TouchEventArgs e)
        {
            btn10.ReleaseTouchCapture(e.TouchDevice);
            btn10.Stroke = brush_nonact_stroke;
            btn10.Fill = brush_nonact_fill;
            joystick.SetBtn(false, id, 10);
        }

        private void btn11_TouchEnter(object sender, TouchEventArgs e)
        {
            btn11.CaptureTouch(e.TouchDevice);
            btn11.Stroke = brush_act_stroke;
            btn11.Fill = brush_act_fill;
            joystick.SetBtn(true, id, 11);
        }

        private void btn11_TouchLeave(object sender, TouchEventArgs e)
        {
            btn11.ReleaseTouchCapture(e.TouchDevice);
            btn11.Stroke = brush_nonact_stroke;
            btn11.Fill = brush_nonact_fill;
            joystick.SetBtn(false, id, 11);
        }

        private void btn12_TouchEnter(object sender, TouchEventArgs e)
        {
            btn12.CaptureTouch(e.TouchDevice);
            btn12.Stroke = brush_act_stroke;
            btn12.Fill = brush_act_fill;
            joystick.SetBtn(true, id, 12);
        }

        private void btn12_TouchLeave(object sender, TouchEventArgs e)
        {
            btn12.ReleaseTouchCapture(e.TouchDevice);
            btn12.Stroke = brush_nonact_stroke;
            btn12.Fill = brush_nonact_fill;
            joystick.SetBtn(false, id, 12);
        }

        
        private void dpad_u_TouchEnter(object sender, TouchEventArgs e)
        {
            dpad_u.Stroke = brush_act_stroke;
            dpad_u.Fill = brush_act_fill;
            joystick.SetDiscPov(0, id, 1);
        }

        private void dpad_u_TouchLeave(object sender, TouchEventArgs e)
        {
            dpad_u.Stroke = brush_nonact_stroke;
            dpad_u.Fill = brush_nonact_fill;
            joystick.SetDiscPov(-1, id, 1);
        }

        private void dpad_l_TouchEnter(object sender, TouchEventArgs e)
        {
            dpad_l.Stroke = brush_act_stroke;
            dpad_l.Fill = brush_act_fill;
            joystick.SetDiscPov(3, id, 1);
        }

        private void dpad_l_TouchLeave(object sender, TouchEventArgs e)
        {
            dpad_l.Stroke = brush_nonact_stroke;
            dpad_l.Fill = brush_nonact_fill;
            joystick.SetDiscPov(-1, id, 1);
        }

        private void dpad_d_TouchEnter(object sender, TouchEventArgs e)
        {
            dpad_d.Stroke = brush_act_stroke;
            dpad_d.Fill = brush_act_fill;
            joystick.SetDiscPov(2, id, 1);
        }

        private void dpad_d_TouchLeave(object sender, TouchEventArgs e)
        {
            dpad_d.Stroke = brush_nonact_stroke;
            dpad_d.Fill = brush_nonact_fill;
            joystick.SetDiscPov(-1, id, 1);
        }

        private void dpad_r_TouchEnter(object sender, TouchEventArgs e)
        {
            dpad_r.Stroke = brush_act_stroke;
            dpad_r.Fill = brush_act_fill;
            joystick.SetDiscPov(1, id, 1);
        }

        private void dpad_r_TouchLeave(object sender, TouchEventArgs e)
        {
            dpad_r.Stroke = brush_nonact_stroke;
            dpad_r.Fill = brush_nonact_fill;
            joystick.SetDiscPov(-1, id, 1);
        }

        // 表示・非表示トグルボタン
        private void btn_menu_TouchUp(object sender, TouchEventArgs e)
        {
            if(show_gamepad)
            {
                // 操作スルー設定
                if(!through_enable)
                {
                    main.Background = brush_through_enable;
                }

                // ボタン表示
                analog_l.Visibility = (analog_l.Visibility == System.Windows.Visibility.Collapsed) ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Hidden;
                analog_r.Visibility = (analog_r.Visibility == System.Windows.Visibility.Collapsed) ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Hidden;
                btn1.Visibility = (btn1.Visibility == System.Windows.Visibility.Collapsed) ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Hidden;
                btn2.Visibility = (btn2.Visibility == System.Windows.Visibility.Collapsed) ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Hidden;
                btn3.Visibility = (btn3.Visibility == System.Windows.Visibility.Collapsed) ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Hidden;
                btn4.Visibility = (btn4.Visibility == System.Windows.Visibility.Collapsed) ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Hidden;
                btn5.Visibility = (btn5.Visibility == System.Windows.Visibility.Collapsed) ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Hidden;
                btn6.Visibility = (btn6.Visibility == System.Windows.Visibility.Collapsed) ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Hidden;
                btn7.Visibility = (btn7.Visibility == System.Windows.Visibility.Collapsed) ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Hidden;
                btn8.Visibility = (btn8.Visibility == System.Windows.Visibility.Collapsed) ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Hidden;
                btn9.Visibility = (btn9.Visibility == System.Windows.Visibility.Collapsed) ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Hidden;
                btn10.Visibility = (btn10.Visibility == System.Windows.Visibility.Collapsed) ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Hidden;
                btn11.Visibility = (btn11.Visibility == System.Windows.Visibility.Collapsed) ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Hidden;
                btn12.Visibility = (btn12.Visibility == System.Windows.Visibility.Collapsed) ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Hidden;
                dpad.Visibility = (dpad.Visibility == System.Windows.Visibility.Collapsed) ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Hidden;
                dpad_ccl.Visibility = (dpad_ccl.Visibility == System.Windows.Visibility.Collapsed) ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Hidden;
                dpad_au.Visibility = (dpad_au.Visibility == System.Windows.Visibility.Collapsed) ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Hidden;
                dpad_al.Visibility = (dpad_al.Visibility == System.Windows.Visibility.Collapsed) ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Hidden;
                dpad_ad.Visibility = (dpad_ad.Visibility == System.Windows.Visibility.Collapsed) ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Hidden;
                dpad_ar.Visibility = (dpad_ar.Visibility == System.Windows.Visibility.Collapsed) ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Hidden;
                dpad_u.Visibility = (dpad_u.Visibility == System.Windows.Visibility.Collapsed) ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Hidden;
                dpad_l.Visibility = (dpad_l.Visibility == System.Windows.Visibility.Collapsed) ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Hidden;
                dpad_d.Visibility = (dpad_d.Visibility == System.Windows.Visibility.Collapsed) ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Hidden;
                dpad_r.Visibility = (dpad_r.Visibility == System.Windows.Visibility.Collapsed) ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Hidden;
            }
            else
            {
                // 操作非スルー設定
                if(!through_enable)
                {
                    main.Background = brush_through_disable;
                }

                // ボタン非表示
                analog_l.Visibility = (analog_l.Visibility == System.Windows.Visibility.Collapsed) ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Visible;
                analog_r.Visibility = (analog_r.Visibility == System.Windows.Visibility.Collapsed) ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Visible;
                btn1.Visibility = (btn1.Visibility == System.Windows.Visibility.Collapsed) ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Visible;
                btn2.Visibility = (btn2.Visibility == System.Windows.Visibility.Collapsed) ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Visible;
                btn3.Visibility = (btn3.Visibility == System.Windows.Visibility.Collapsed) ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Visible;
                btn4.Visibility = (btn4.Visibility == System.Windows.Visibility.Collapsed) ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Visible;
                btn5.Visibility = (btn5.Visibility == System.Windows.Visibility.Collapsed) ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Visible;
                btn6.Visibility = (btn6.Visibility == System.Windows.Visibility.Collapsed) ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Visible;
                btn7.Visibility = (btn7.Visibility == System.Windows.Visibility.Collapsed) ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Visible;
                btn8.Visibility = (btn8.Visibility == System.Windows.Visibility.Collapsed) ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Visible;
                btn9.Visibility = (btn9.Visibility == System.Windows.Visibility.Collapsed) ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Visible;
                btn10.Visibility = (btn10.Visibility == System.Windows.Visibility.Collapsed) ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Visible;
                btn11.Visibility = (btn11.Visibility == System.Windows.Visibility.Collapsed) ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Visible;
                btn12.Visibility = (btn12.Visibility == System.Windows.Visibility.Collapsed) ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Visible;
                dpad.Visibility = (dpad.Visibility == System.Windows.Visibility.Collapsed) ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Visible;
                dpad_ccl.Visibility = (dpad_ccl.Visibility == System.Windows.Visibility.Collapsed) ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Visible;
                dpad_au.Visibility = (dpad_au.Visibility == System.Windows.Visibility.Collapsed) ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Visible;
                dpad_al.Visibility = (dpad_al.Visibility == System.Windows.Visibility.Collapsed) ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Visible;
                dpad_ad.Visibility = (dpad_ad.Visibility == System.Windows.Visibility.Collapsed) ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Visible;
                dpad_ar.Visibility = (dpad_ar.Visibility == System.Windows.Visibility.Collapsed) ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Visible;
                dpad_u.Visibility = (dpad_u.Visibility == System.Windows.Visibility.Collapsed) ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Visible;
                dpad_l.Visibility = (dpad_l.Visibility == System.Windows.Visibility.Collapsed) ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Visible;
                dpad_d.Visibility = (dpad_d.Visibility == System.Windows.Visibility.Collapsed) ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Visible;
                dpad_r.Visibility = (dpad_r.Visibility == System.Windows.Visibility.Collapsed) ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Visible;
            }
            show_gamepad = !show_gamepad;
        }
    }
}
